(* Autor: Miloslav Beta
   Program: Demostrace transformanch algoritm pro pevod
   bezkontextovch gramatik do normlnch tvar
   Copyright (c) 1993
*)


program DemoGram;

{$M 16384,16384,655360}
{$X+,S-}

uses Dos, Objects, Drivers, Memory, Views, Menus, Dialogs,
  StdDlg, MsgBox, App, Buffers, Editor, Common, Pol, Grammar,
  BigGram, MyApp, Tracer, DemoChom, DemoRec, DemoGrei, DemoJedn,
  HelpFile, DGHelp, MyTracer;

const
  HeapSize = 64 * (1024 div 16);

const
  cmOpen          = 100;
  cmNew           = 101;
  cmChangeDir     = 102;
  cmDosShell      = 103;
  cmShowClip      = 104;

const
  cmVJednoduche   = 1000;
  cmVChomskeho    = 1001;
  cmVRekurze      = 1002;
  cmVGreibachova  = 1003;
  cmObsah         = 1004;
  cmIndex         = 1005;
  cmInformace     = 1006;
  cmBackSteps     = 1007;

type
  PEditorApp = ^TEditorApp;
  TEditorApp = object(TApplication)
    Clock: PMyClock;

    constructor Init;
    destructor Done; virtual;
    procedure GetEvent(var Event: TEvent);virtual;
    function  GetPalette: PPalette; virtual;
    procedure HandleEvent(var Event: TEvent); virtual;
    procedure InitMenuBar; virtual;
    procedure InitStatusLine; virtual;
    procedure OutOfMemory; virtual;
    procedure Idle; virtual;
  end;

var
  EditorApp: TEditorApp;
  ClipWindow: PEditWindow;

function CalcHelpName: PathStr;
var
  EXEName: PathStr;
  Dir: DirStr;
  Name: NameStr;
  Ext: ExtStr;
begin
  if Lo(DosVersion) >= 3 then EXEName := ParamStr(0)
  else EXEName := FSearch('dg.exe', GetEnv('PATH'));
  FSplit(EXEName, Dir, Name, Ext);
  if Dir[Length(Dir)] = '\' then Dec(Dir[0]);
  CalcHelpName := FSearch('DGHELP.HLP', Dir);
end;

procedure TEditorApp.GetEvent(var Event: TEvent);
var
  W: PWindow;
  HFile: PHelpFile;
  HelpStrm: PDosStream;
  i:integer;
const
  HelpInUse: Boolean = False;
begin
  TApplication.GetEvent(Event);
  case Event.What of
    evCommand:
      if ((Event.Command=cmHelp) or (Event.Command=cmLastHelpTopic))
         and not HelpInUse then
      begin
        HelpInUse := True;
        HelpStrm := New(PDosStream, Init(CalcHelpName, stOpenRead));
        HFile := New(PHelpFile, Init(HelpStrm));
        if HelpStrm^.Status <> stOk then
        begin
          MessageBox('Soubor npovdy nebyl nalezen.', nil, mfError + mfOkButton);
          Dispose(HFile, Done);
        end
        else
        begin
          if Event.Command=cmHelp then
            W := New(PHelpWindow,Init(HFile, 'Npovda', GetHelpCtx))
          else
            W := New(PHelpWindow,Init(HFile, 'Npovda', hcLastTopic));
          if ValidView(W) <> nil then
          begin
            ExecView(W);
            Dispose(W, Done);
          end;
          ClearEvent(Event);
        end;
        HelpInUse := False;
      end;
  end;
end;

function TEditorApp.GetPalette: PPalette;
const
  CNewColor = CColor + CHelpColor;
  CNewBlackWhite = CBlackWhite + CHelpBlackWhite;
  CNewMonochrome = CMonochrome + CHelpMonochrome;
  P: array[apColor..apMonochrome] of string[Length(CNewColor)] =
    (CNewColor, CNewBlackWhite, CNewMonochrome);
begin
  GetPalette := @P[AppPalette];
end;

constructor TEditorApp.Init;
var
  H: Word;
  R: TRect;
  I: Integer;
  FileName: PathStr;
  s:string;
  mem:word;
  j:integer;
begin
  NotEnoughMemory:=MyLowMem;
  InitHashTab;
  mem:=0;
  for i:=1 to ParamCount do begin
    s:=ParamStr(i);
    if (s[1]='/') and ((s[2]='m')or(s[2]='M')) then begin
      system.delete(s,1,2);
      val(s,mem,j);
      if j<>0 then mem:=0;
    end;
  end;
  if mem>=20 then mem:=mem * (1024 div 16)
  else mem:=HeapSize;
  H := PtrRec(HeapEnd).Seg - PtrRec(HeapPtr).Seg;
  if H<mem+32*(1024 div 16) then mem:=H div 2;
  BufHeapSize := mem;
  InitBuffers;
  TApplication.Init;
  DisableCommands([cmSave, cmSaveAs, cmCut, cmCopy, cmPaste, cmClear,
    cmUndo, cmFind, cmReplace, cmSearchAgain, cmTNadbytecne, cmTNedostupne,
    cmTRedukovana, cmTBezEpsilon, cmTCykly, cmTVlastni, cmTJednoduche,
    cmTBezRekurze, cmTChomskeho, cmTGreibachova, cmTZjednoduseni,
    cmDJednoduche, cmDChomskeho, cmDRekurze, cmDGreibachova, cmReadGrammar,
    cmWriteGrammar, cmUkazStav]);
  EditorDialog := DoEditDialog;
  ClipWindow := OpenEditor('', False);
  if ClipWindow<>nil then begin
    Clipboard:=ClipWindow^.Editor;
    Clipboard^.CanUndo:=False;
  end;

  RegisterHelpFile;

  GetExtent(R);
  R.A.X := R.B.X - 10; R.B.Y := R.A.Y + 1;
  Clock:=New(PMyClock, Init(R));
  Insert(Clock);

  Message(@Self,evBroadcast,cmBusy,nil);

  for I := 1 to ParamCount do
  begin
    FileName := ParamStr(I);
    if Filename[1]<>'/' then begin
      if FileName[Length(FileName)] = '\' then
        FileName := FileName + '*.grm';
      if (Pos('?', FileName) = 0) and (Pos('*', FileName) = 0) then
        OpenEditor(FExpand(FileName),true)
      else
        if ExecDialog(New(PFileDialog, Init('*.grm', 'Oteven souboru',
                '~J~mno', fdOpenButton, 100)), @FileName) <> cmCancel then
          OpenEditor(FileName, True);
    end;
  end;
end;

destructor TEditorApp.Done;
begin
  TApplication.Done;
  DoneBuffers;
  dispose(Clock,Done);
  DoneHashTab;
end;

procedure TEditorApp.HandleEvent(var Event: TEvent);
const w : PCompileDialog = nil;

 procedure Start;
 var Editor:PFileEditor;
 begin
   Editor:=PMyEditWindow(Message(DeskTop,evCommand,cmEditor,nil))^.Editor;
   w := New(PCompileDialog, Init(Editor^.FileName));
   DeskTop^.Insert(w);
 end;

 procedure Konec;
 begin
   if w<>nil then begin
     DeskTop^.Delete(w);
     dispose(w,Done);
     w:=nil;
   end;
 end;

 procedure FileOpen;
 var
   FileName: FNameStr;
   Dialog: PDialog;
 begin
   FileName := '*.grm';
   Dialog:=New(PFileDialog, Init('*.grm', 'Oteven souboru',
     '~J~mno', fdOpenButton, 100));
   if ExecDialog(Dialog,@FileName) <> cmCancel then
     OpenEditor(FileName, True);
 end;

 procedure FileNew;
 begin
   OpenEditor('', True);
 end;

 procedure ChangeDir;
 begin
   ExecDialog(New(PChDirDialog, Init(cdNormal, 0)), nil);
 end;

 procedure DosShell;
 begin
   DoneSysError;
   DoneEvents;
   DoneVideo;
   DoneMemory;
   SetMemTop(Ptr(BufHeapPtr, 0));
   PrintStr('Pkaz EXIT zpsob nvrat...');
   SwapVectors;
   Exec(GetEnv('COMSPEC'), '');
   SwapVectors;
   SetMemTop(Ptr(BufHeapEnd, 0));
   InitMemory;
   InitVideo;
   InitEvents;
   InitSysError;
   Redraw;
 end;

 procedure ShowClip;
 begin
   ClipWindow^.Select;
   ClipWindow^.Show;
 end;

 procedure Tile;
 var
   R: TRect;
 begin
   Desktop^.GetExtent(R);
   Desktop^.Tile(R);
 end;

 procedure Cascade;
 var
   R: TRect;
 begin
   Desktop^.GetExtent(R);
   Desktop^.Cascade(R);
 end;

 procedure Err(s:string;hlp:word);
 var err:TError;
     pom:longint;
     Editor:PFileEditor;
 begin
   Editor:=PMyEditWindow(Message(DeskTop,evCommand,cmEditor,nil))^.Editor;
   with Editor^ do begin
     ErrorBox(s,hlp);
     err:=TError(Event.InfoPtr^);
     with err do begin
       SetSelect(n,n,false);
       pom := r - Size.Y div 2;
       if pom<0 then r:=0 else r:=pom;
       if s>Size.X-5 then pom:=s-Size.X+15
       else pom:=0;
       if pom>s then pom:=s-5;
       if pom<0 then s:=0 else s:=pom;
       ScrollTo(s,r);
     end;
   end;
 end;

 procedure WriteGrammar(Gram:PBigGrammar);
 var
   R: TRect;
   Dest:PMyEditWindow;
   Novy:boolean;
   s:string;
 begin
   if Gram=nil then exit;
   s:=Gram^.Name;
   if length(s)>8 then system.Delete(s,9,length(s)-8);
   s:=s+'.gra';Novy:=false;
   Dest:=Message(DeskTop,evBroadcast,cmGetTitle,@s);
   if Dest=nil then begin
     DeskTop^.GetExtent(R);
     Dest:=PMyEditWindow(Application^.ValidView(New(PMyEditWindow,
       Init(R,s,wnNoNumber))));
     if Dest=nil then exit;
     Novy:=true;
   end;
   with Dest^ do begin
     Lock;
     with Dest^.Editor^ do begin
       SetSelect(0,BufLen,false);
       DeleteSelect;
       Gram^.WriteGrammar(Dest^.Editor);
       SetSelect(0,0,false);
     end;
     if Novy then DeskTop^.Insert(Dest)
     else begin
       Editor^.Ready:=false;
       Select;
     end;
     Unlock;
   end;
 end;

 procedure Vyklad(s:string;t:word);
 var
   W: PWindow;
   HFile: PHelpFile;
   HelpStrm: PDosStream;
 begin
   HelpStrm := New(PDosStream, Init(CalcHelpName, stOpenRead));
   HFile := New(PHelpFile, Init(HelpStrm));
   if HelpStrm^.Status <> stOk then
   begin
     MessageBox('Soubor npovdy nebyl nalezen.', nil, mfError + mfOkButton);
     Dispose(HFile, Done);
   end
   else
   begin
     W := New(PHelpWindow,Init(HFile, s, t));
     if ValidView(W) <> nil then
     begin
       DeskTop^.ExecView(W);
       Dispose(W, Done);
     end;
   end;
 end;

 function GetActGrammar:PGrammar;
 var
   Gramatika:PGrammar;
   P:PMyEditWindow;
   Q:pointer;
 begin
   Gramatika:=nil;
   P:=PMyEditWindow(Message(DeskTop,evCommand,cmEditor,nil));
   if (P=nil) or (P=PMyEditWindow(ClipWindow)) then begin
     Q:=Message(DeskTop,evCommand,cmDemoWindow,nil);
     if Q<>nil then
       Message(Q,evCommand, cmGramatika,@Gramatika);
   end else begin
     if not P^.Editor^.Ready then
       Message(P,evCommand,cmReadGrammar,nil);
     if not P^.Editor^.Ready then exit else Gramatika:=P^.Gramatika;
   end;
   if Gramatika<>nil then GetActGrammar:=Gramatika^.GetNewCopy
   else GetActGrammar:=nil;
 end;

 procedure Busy;
 begin
  Message(@Self,evBroadcast,cmBusy,nil);
 end; 

 procedure Uprava(Gramatika:PGrammar;s1,s2:string;command:word);
 var
   Dialog: PDialog;
   R: TRect;
   C: Word;
 begin
   R.Assign(25,6,55,18);
   Dialog := New(PDialog, Init(R, 'Upozornn'));
   with Dialog^ do
   begin
     HelpCtx:=hcPredUprava;
     R.Assign(14-length(s1)div 2, 2, 29, 3);
     Insert(New(PLabel, Init(R, s1, nil)));
     if s2<>'' then begin
       R.Assign(14-length(s2)div 2,3,29,4);
       Insert(New(PLabel, Init(R, s2, nil)));
       R.Assign(2, 5, 28, 6);
     end else R.Assign(2,3,28,4);
     Insert(New(PLabel, Init(R, 'Chcete gramatiku upravit?', nil)));

     R.Assign(3, Size.Y - 4,12 , Size.Y - 2);
     Insert(New(PButton, Init(R, '~A~no', cmYes, bfDefault)));
     Inc(R.A.X, 12); Inc(R.B.X, 12);
     Insert(New(PButton, Init(R, '~N~e', cmNo, bfNormal)));
     SelectNext(False);
   end;
   C := DeskTop^.ExecView(Dialog);
   Dispose(Dialog, Done);
   if C = cmYes then begin
     Busy;
     case command of
       cmTBezEpsilon   : Gramatika^.ZrusEpsilonovaPravidla;
       cmTVlastni      : Gramatika^.VytvorVlastni;
       cmTJednoduche   : begin
                           Gramatika^.VytvorVlastni;
                           Gramatika^.ZrusJednoduchaPravidla;
                         end;
       cmTBezRekurze   : begin
                           Gramatika^.VytvorVlastni;
                           Gramatika^.VytvorBezRekurze;
                         end;
     end;
   end;
 end;

 procedure DemoChomskeho;
 var
   Gramatika:PGrammar;
 begin
   Gramatika:=GetActGrammar;
   if Gramatika=nil then exit;
   if not Gramatika^.PlatiZe(_jeVlastni_) or
      not Gramatika^.PlatiZe(_jeJednoducha_) then
     Uprava(Gramatika,'Gramatika nen vlastn','bez jednoduchch pravidel',
       cmTJednoduche);
   if Gramatika^.PlatiZe(_jeVlastni_) and Gramatika^.PlatiZe(_jeJednoducha_)
   then
     DeskTop^.Insert(Application^.ValidView(new(PDemoChom,Init(Gramatika,GetWinNum))))
   else dispose(Gramatika,Done);
 end;

 procedure DemoRec;
 var
   Gramatika:PGrammar;
 begin
   Gramatika:=GetActGrammar;
   if Gramatika=nil then exit;
   if not Gramatika^.PlatiZe(_jeVlastni_) then
     Uprava(Gramatika,'Gramatika nen vlastn','',cmTVlastni);
   if Gramatika^.PlatiZe(_jeVlastni_) then
     DeskTop^.Insert(Application^.ValidView(new(PDemoRec,Init(Gramatika,GetWinNum))))
   else dispose(Gramatika,Done);
 end;

 procedure DemoGreibachova;
 var
   Gramatika:PGrammar;
 begin
   Gramatika:=GetActGrammar;
   if Gramatika=nil then exit;
   if not Gramatika^.PlatiZe(_jeVlastni_) or
      not Gramatika^.PlatiZe(_jeBezRekurze_) then
     Uprava(Gramatika,'Gramatika nen vlastn','bez lev rekurze',
       cmTBezRekurze);
   if Gramatika^.PlatiZe(_jeVlastni_) and  Gramatika^.PlatiZe(_jeBezRekurze_) then
     DeskTop^.Insert(Application^.ValidView(new(PDemoGrei,Init(Gramatika,GetWinNum))))
   else dispose(Gramatika,Done);
 end;

 procedure DemoJedn;
 var
   Gramatika:PGrammar;
 begin
   Gramatika:=GetActGrammar;
   if Gramatika=nil then exit;
   if not Gramatika^.PlatiZe(_jeBezEpsilon_) then
     Uprava(Gramatika,'Gramatika nen bez','epsilonovch pravidel',
      cmTBezEpsilon);
   if Gramatika^.PlatiZe(_jeBezEpsilon_) then
     DeskTop^.Insert(Application^.ValidView(new(PDemoJedn,Init(Gramatika,GetWinNum))))
   else dispose(Gramatika,Done);
 end;

 procedure Informace;
 var R:TRect;
 begin
   R.Assign(0, 0, 30, 17);
   R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
   MessageBoxRect(R,
          'Demonstrace transformace'#13#13+
          'bezkontextovch gramatik'#13#13+
          '  do normlnch forem'#13#13#13+
          '   Copyright (c) 1993 '#13#13+
          '     Miloslav Beta',
     nil,mfInformation+mfOkButton);
 end;

begin
  TApplication.HandleEvent(Event);
  case Event.What of
    evCommand:
      case Event.Command of
        cmOpen          : begin Busy;FileOpen;end;
        cmNew           : begin Busy;FileNew;end;
        cmChangeDir     : begin Busy;ChangeDir;end;
        cmDosShell      : begin Busy;DosShell;end;
        cmShowClip      : begin Busy;ShowClip;end;
        cmTile          : begin Busy;Tile;end;
        cmCascade       : begin Busy;Cascade;end;
        cmVJednoduche   : Vyklad('Vklad algoritm',hcAlgJednoducha);
        cmVChomskeho    : Vyklad('Vklad algoritm',hcAlgCNF);
        cmVRekurze      : Vyklad('Vklad algoritm',hcAlgLevaRekurze);
        cmVGreibachova  : Vyklad('Vklad algoritm',hcAlgGNF);
        cmObsah         : Vyklad('Obsah',hcObsah);
        cmIndex         : Vyklad('Index',hcIndex);
        cmInformace     : Informace;
        cmBackSteps     : begin Busy;SetMaxNumOfBackSteps;end;
        cmDJednoduche   : begin Busy;DemoJedn;end;
        cmDChomskeho    : begin Busy;DemoChomskeho;end;
        cmDRekurze      : begin Busy;DemoRec;end;
        cmDGreibachova  : begin Busy;DemoGreibachova;end;
        cmWriteGrammar  : begin Busy;WriteGrammar(PBigGrammar(Event.InfoPtr));end;
        else exit;     
      end;
    evMyMessage :
      case Event.Command of
        msNeocekavanyKonec   : Err('Neoekvan konec souboru',hce1);
        msGrammar            : Err('Oekvm klov slovo GRAMMAR',hce2);
        msIdentifikator      : Err('Oekvm identifiktor',hce3);
        msLevaKulataZavorka  : Err('Oekvm (',hce4);
        msPravaKulataZavorka : Err('Oekvm )',hce5);
        msLevaSlozZavorka    : Err('Oekvm {',hce6);
        msPravaSlozZavorka   : Err('Oekvm }',hce7);
        msTecka              : Err('Oekvm .',hce8);
        msCarka              : Err('Oekvm ,',hce9);
        msStrednik           : Err('Oekvm ;',hce10);
        msRovnitko           : Err('Oekvm =',hce11);
        msSpatnyInterval     : Err('Chybn zadan interval',hce12);
        msTerminal     : Err('Oekvm terminln symbol',hce13);
        msNeterminal   : Err('Oekvm neterminln symbol',hce14);
        msSymbol       : Err('Oekvm terminln nebo neterminln symbol',hce15);
        msIdTerm       : Err('Oekvm identifiktor mnoiny terminl',hce16);
        msIdNeterm     : Err('Oekvm identifiktor mnoiny neterminl',hce17);
        msIdPrav       : Err('Oekvm identifiktor mnoiny pravidel',hce18);
        msSipka        : Err('Oekvm -->',hce19);
        msEpsilon      : Err('Epsilon nesm bt terminln ani neterminln symbol',hce20);
        msMeziterminal : Err('V intervalu se vyskytuje neterminln symbol',hce21);

        msStart        : Start;
        msEnd          : Konec;
	else exit;
      end;
    else
      Exit;
  end;
  ClearEvent(Event);
end;

procedure TEditorApp.InitMenuBar;
var
  R: TRect;
begin
  GetExtent(R);
  R.B.Y := R.A.Y + 1;
  MenuBar := New(PMenuBar, Init(R, NewMenu(
    NewSubMenu('~S~oubor', hcSoubor, NewMenu(
      NewItem('~O~tevi...', 'F3', kbF3, cmOpen, hcSouborOtevri,
      NewItem('~N~ov', '', kbNoKey, cmNew, hcSouborNovy,
      NewItem('~U~lo', 'F2', kbF2, cmSave, hcSouborUloz,
      NewItem('Ulo j~a~ko...', '', kbNoKey, cmSaveAs, hcSouborUlozJako,
      NewLine(
      NewItem('~P~eklad', 'Alt-F9', kbAltF9, cmReadGrammar, hcSouborPreklad,
      NewItem('~Z~pis', 'Ctrl-F2', kbCtrlF2, cmDoWrite, hcSouborZapis,
      NewItem('~S~tav...', 'Ctrl-F4', kbCtrlF4, cmUkazStav, hcSouborStav,
      NewLine(
      NewItem('Z~m~na adr...', '', kbNoKey, cmChangeDir, hcSouborZmenaAdr,
      NewItem('~D~OS shell', '', kbNoKey, cmDosShell, hcSouborDosShell,
      NewItem('~K~onec', 'Alt-K', kbAltK, cmQuit, hcSouborKonec,
      nil))))))))))))),
    NewSubMenu('~E~ditor', hcEditor, NewMenu(
      NewItem('~O~bnov dek', '', kbNoKey, cmUndo, hcEditorRadek,
      NewLine(
      NewItem('~S~thej', 'Shift-Del', kbShiftDel, cmCut, hcEditorStrihej,
      NewItem('~K~opruj', 'Ctrl-Ins', kbCtrlIns, cmCopy, hcEditorKopiruj,
      NewItem('~V~lo', 'Shift-Ins', kbShiftIns, cmPaste, hcEditorVloz,
      NewItem('~U~ka clipboard', '', kbNoKey, cmShowClip, hcEditorClipboard,
      NewLine(
      NewItem('Vy~m~a', 'Ctrl-Del', kbCtrlDel, cmClear, hcEditorVymaz,
      nil))))))))),
    NewSubMenu('~H~ledej', hcHledej, NewMenu(
      NewItem('~H~ledej...', '', kbNoKey, cmFind, hcHledejHledej,
      NewItem('~N~ahra...', '', kbNoKey, cmReplace, hcHledejNahrad,
      NewItem('Hledej ~z~novu', '', kbNoKey, cmSearchAgain, hcHledejZnovu,
      nil)))),
    NewSubMenu('~O~kna', hcOkna, NewMenu(
      NewItem('~R~ozmr','Ctrl-F5', kbCtrlF5, cmResize, hcOknaRozmer,
      NewItem('~Z~vten', 'F5', kbF5, cmZoom, hcOknaZvetseni,
      NewItem('~M~ozaika', '', kbNoKey, cmTile, hcOknaMozaika,
      NewItem('~S~tupn', '', kbNoKey, cmCascade, hcOknaStupne,
      NewItem('~D~al', 'F6', kbF6, cmNext, hcOknaDalsi,
      NewItem('~P~edchoz', 'Shift-F6', kbShiftF6, cmPrev, hcOknaPredchozi,
      NewItem('~U~zaven', 'Alt-F3', kbAltF3, cmClose, hcOknaUzavreni,
      nil)))))))),
    NewSubMenu('~D~emo',hcDemo,NewMenu(
      NewItem('~J~ednoduch', 'Alt-J', kbAltJ, cmDJednoduche, hcDJednoduche,
      NewItem('~C~homskho', 'Alt-C', kbAltC, cmDChomskeho, hcDChomskeho,
      NewItem('~R~ekurze', 'Alt-R', kbAltR, cmDRekurze, hcDRekurze,
      NewItem('~G~reibachova', 'Alt-G', kbAltG, cmDGreibachova,hcDGreibachova,
      NewLine(
      NewItem('~V~ped',  'F8', kbF8, cmDoStepF8, hcDVpred,
      NewItem('~Z~pt', 'F9', kbF9, cmUnDoStep, hcDZpet,
      NewItem('~A~uto/Stop', 'Ctrl-F9', kbCtrlF9, cmTraceAlg, hcDAuto,
      NewItem('Jdi ~n~a...', 'F4', kbF4, cmGoTo, hcDJdiNa,
      NewItem('~P~eruen...', 'Ctrl-F8', kbCtrlF8, cmSetBreakPoint, hcDPreruseni,
      NewLine(
      NewItem('Na~s~taven...', '', kbNoKey, cmBackSteps, hcDNastaveni,
      nil))))))))))))),
    NewSubMenu('~U~pravy', hcUpravy, NewMenu(
      NewItem('Na~d~byten', '', kbNoKey, cmTNadbytecne, hcUNadbytecne,
      NewItem('~N~edostupn', '', kbNoKey, cmTNedostupne, hcUNedostupne,
      NewItem('~R~edukovan', '', kbNoKey, cmTRedukovana, hcURedukovana,
      NewItem('~E~psilon', '', kbNoKey, cmTBezEpsilon, hcUEpsilon,
      NewItem('C~y~kly', '', kbNoKey, cmTCykly, hcUCykly,
      NewItem('~V~lastn', '', kbNoKey, cmTVlastni, hcUVlastni,
      NewItem('~J~ednoduch', '', kbNoKey, cmTJednoduche, hcUJednoduche,
      NewItem('~L~ev rekurze', '', kbNoKey, cmTBezRekurze, hcURekurze,
      NewItem('~C~homskho', '', kbNoKey, cmTChomskeho, hcUChomskeho,
      NewItem('~G~reibachova', '', kbNoKey, cmTGreibachova, hcUGreibachova,
      NewLine(
      NewItem('~Z~jednoduen', '', kbNoKey, cmTZjednoduseni, hcUZjednoduseni,
      nil))))))))))))),
    NewSubMenu('~V~klad', hcVyklad, NewMenu(
      NewItem('~J~ednoduch', '', kbNoKey, cmVJednoduche, hcVJednoduche,
      NewItem('~C~homskho', '', kbNoKey, cmVChomskeho, hcVChomskeho,
      NewItem('~R~ekurze', '', kbNoKey, cmVRekurze, hcVRekurze,
      NewItem('~G~reibachova', '', kbNoKey, cmVGreibachova,hcVGreibachova,
      nil))))),
    NewSubMenu('~N~povda', hcNapoveda, NewMenu(
      NewItem('~O~bsah', '', kbNoKey, cmObsah, hcNObsah,
      NewItem('~I~ndex', '', kbNoKey, cmIndex, hcNIndex,
      NewItem('In~f~ormace...', '', kbNoKey, cmInformace, hcNInformace,
      nil)))),
    nil)))))))))));
end;

procedure TEditorApp.InitStatusLine;
var
  R: TRect;
  P: PStatusDef;
begin
  GetExtent(R);
  R.A.Y := R.B.Y - 1;
  P:=NewStatusDef(hcUpravy,hcUpravy,
      NewStatusKey('~F1~ Npovda  Pkazy pro pravu gramatiky do rznch tvar',kbF1,cmHelp,nil),
    NewStatusDef(hcUNadbytecne,hcUNadbytecne,
      NewStatusKey('~F1~ Npovda  prava na gramatiku bez nadbytench symbol',kbF1,cmHelp,nil),
    NewStatusDef(hcUNedostupne,hcUNedostupne,
      NewStatusKey('~F1~ Npovda  prava na gramatiku bez nedostupnch symbol',kbF1,cmHelp,nil),
    NewStatusDef(hcURedukovana,hcURedukovana,
      NewStatusKey('~F1~ Npovda  prava na redukovanou gramatiku',kbF1,cmHelp,nil),
    NewStatusDef(hcUEpsilon,hcUEpsilon,
      NewStatusKey('~F1~ Npovda  prava na gramatiku bez epsilonovch pravidel',kbF1,cmHelp,nil),
    NewStatusDef(hcUCykly,hcUCykly,
      NewStatusKey('~F1~ Npovda  prava na gramatiku bez cykl',kbF1,cmHelp,nil),
    NewStatusDef(hcUVlastni,hcUVlastni,
      NewStatusKey('~F1~ Npovda  pava na vlastn gramatiku',kbF1,cmHelp,nil),
    NewStatusDef(hcUJednoduche,hcUJednoduche,
      NewStatusKey('~F1~ Npovda  prava na gramatiku bez jednoduchch pravidel',kbF1,cmHelp,nil),
    NewStatusDef(hcUChomskeho,hcUChomskeho,
      NewStatusKey('~F1~ Npovda  prava na gramatiku v Chomskho normln form',kbF1,cmHelp,nil),
    NewStatusDef(hcURekurze,hcURekurze,
      NewStatusKey('~F1~ Npovda  prava na gramatiku bez lev rekurze',kbF1,cmHelp,nil),
    NewStatusDef(hcUGreibachova,hcUGreibachova,
      NewStatusKey('~F1~ Npovda  prava na gramatiku v Greibachov normln form',kbF1,cmHelp,nil),
    NewStatusDef(hcUZjednoduseni,hcUZjednoduseni,
      NewStatusKey('~F1~ Npovda  Zjednoduen neterminlnch symbol aktuln gramatiky',kbF1,cmHelp,nil),
    NewStatusDef(hcVyklad,hcVyklad,
      NewStatusKey('~F1~ Npovda  Pkazy pro vklad transformanch algoritm',kbF1,cmHelp,nil),
    NewStatusDef(hcVJednoduche,hcVJednoduche,
      NewStatusKey('~F1~ Npovda  Alg. pravy na gramatiku bez jednoduchch pravidel',kbF1,cmHelp,nil),
    NewStatusDef(hcVChomskeho,hcVChomskeho,
      NewStatusKey('~F1~ Npovda  Alg. pravy na gramatiku v Chomskho normln form',kbF1,cmHelp,nil),
    NewStatusDef(hcVRekurze,hcVRekurze,
      NewStatusKey('~F1~ Npovda  Alg. pravy na gramatiku bez lev rekurze',kbF1,cmHelp,nil),
    NewStatusDef(hcVGreibachova,hcVGreibachova,
      NewStatusKey('~F1~ Npovda  Alg. pravy na gramatiku v Greibachov normln form',kbF1,cmHelp,nil),
    NewStatusDef(hcNapoveda,hcNapoveda,
      NewStatusKey('~F1~ Npovda  Pkazy pro volbu npovdy',kbF1,cmHelp,nil),
    NewStatusDef(hcNObsah,hcNObsah,
      NewStatusKey('~F1~ Npovda  Obsah souboru npovdy',kbF1,cmHelp,nil),
    NewStatusDef(hcNIndex,hcNIndex,
      NewStatusKey('~F1~ Npovda  Index vech hesel souboru npovdy',kbF1,cmHelp,nil),
    NewStatusDef(hcNInformace,hcNInformace,
      NewStatusKey('~F1~ Npovda  Informace o nzvu programu a copyrightu',kbF1,cmHelp,nil),
    NewStatusDef(hcAlgWindow,hcAlgWindow,
      NewStatusKey('~F8~ Vped', kbF8, cmDoStepF8,
      NewStatusKey('~F9~ Zpt', kbF9, cmUnDoStep,
      NewStatusKey('~F4~ Jdi na', kbF4, cmGoTo,
      NewStatusKey('~Ctrl-F8~ Peruen', kbCtrlF8, cmSetBreakPoint,
      NewStatusKey('~Ctrl-F9~ Auto/Stop', kbNoKey, cmTraceAlg,
      NewStatusKey('~F10~ Menu', kbF10, cmMenu,
      NewStatusKey('', kbCtrlF5, cmResize,
      NewStatusKey('', kbF1, cmHelp,
      NewStatusKey('', kbAltF1, cmLastHelpTopic,
      nil))))))))),
    NewStatusDef(hcNoContext,$FFFF,
      NewStatusKey('~F3~ Otevi', kbF3, cmOpen,
      NewStatusKey('~F2~ Ulo', kbF2, cmSave,
      NewStatusKey('~Ctrl-F4~ Stav', kbCtrlF4, cmUkazStav,
      NewStatusKey('~F5~ Zvti', kbF5, cmZoom,
      NewStatusKey('~F6~ Dal', kbF6, cmNext,
      NewStatusKey('~Alt-K~ Konec', kbAltK, cmQuit,
      NewStatusKey('~F10~ Menu', kbF10, cmMenu,
      NewStatusKey('', kbCtrlF5, cmResize,
      NewStatusKey('', kbF1, cmHelp,
      NewStatusKey('', kbAltF1, cmLastHelpTopic,
      nil)))))))))),
    nil)))))))))))))))))))))));
  New(StatusLine, Init(R,
    NewStatusDef(hcDragging,hcDragging,
      NewStatusKey('~'#24#25#26#27'~ Pesun  ~Shitf-'#24#25#26#27'~ Rozmry  ~~ Hotovo  ~Esc~ Zruit',kbNoKey,cmValid,nil),
    NewStatusDef(hcSoubor,hcSoubor,
      NewStatusKey('~F1~ Npovda  Pkazy pro prci se soubory',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborOtevri,hcSouborOtevri,
      NewStatusKey('~F1~ Npovda  Najde a oteve soubor v editanm okn',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborNovy,hcSouborNovy,
      NewStatusKey('~F1~ Npovda  Oteve nov soubor v editanm okn',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborUloz,hcSouborUloz,
      NewStatusKey('~F1~ Npovda  Ulo editovan soubor na disk',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborUlozJako,hcSouborUlozJako,
      NewStatusKey('~F1~ Npovda  Ulo editovan soubor na disk pod novm jmnem',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborPreklad,hcSouborPreklad,
      NewStatusKey('~F1~ Npovda  Peklad editovan gramatiky do vnitnho tvaru',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborZapis,hcSouborZapis,
      NewStatusKey('~F1~ Npovda  Zpis aktuln gramatiky z vnitnho tvaru do souboru',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborStav,hcSouborStav,
      NewStatusKey('~F1~ Npovda  Zobrazen stavu aktuln gramatiky',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborZmenaAdr,hcSouborZmenaAdr,
      NewStatusKey('~F1~ Npovda  Zmna aktulnho adrese',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborDosShell,hcSouborDosShell,
      NewStatusKey('~F1~ Npovda  Odskok do operanho systmu',kbF1,cmHelp,nil),
    NewStatusDef(hcSouborKonec,hcSouborKonec,
      NewStatusKey('~F1~ Npovda  Konec prce s programem',kbF1,cmHelp,nil),
    NewStatusDef(hcEditor,hcEditor,
      NewStatusKey('~F1~ Npovda  Pkazy pro prci s textovm editorem',kbF1,cmHelp,nil),
    NewStatusDef(hcEditorRadek,hcEditorRadek,
      NewStatusKey('~F1~ Npovda  Obnoven posledn editovanho dku do pvodnho stavu',kbF1,cmHelp,nil),
    NewStatusDef(hcEditorStrihej,hcEditorStrihej,
      NewStatusKey('~F1~ Npovda  Vystien bloku textu z editovanho souboru do clipboardu',kbF1,cmHelp,nil),
    NewStatusDef(hcEditorVloz,hcEditorVloz,
      NewStatusKey('~F1~ Npovda  Vloen bloku textu z clipboardu do editovanho souboru',kbF1,cmHelp,nil),
    NewStatusDef(hcEditorKopiruj,hcEditorKopiruj,
      NewStatusKey('~F1~ Npovda  Zkoprovn bloku textu z editovanho souboru na clipboard',kbF1,cmHelp,nil),
    NewStatusDef(hcEditorClipboard,hcEditorClipboard,
      NewStatusKey('~F1~ Npovda  Zobrazen okna clipboardu',kbF1,cmHelp,nil),
    NewStatusDef(hcEditorVymaz,hcEditorVymaz,
      NewStatusKey('~F1~ Npovda  Smazn oznaenho bloku textu',kbF1,cmHelp,nil),
    NewStatusDef(hcHledej,hcHledej,
      NewStatusKey('~F1~ Npovda  Pkazy pro prci s etzci',kbF1,cmHelp,nil),
    NewStatusDef(hcHledejHledej,hcHledejHledej,
      NewStatusKey('~F1~ Npovda  Hledn zadanho etzce v editovanm souboru',kbF1,cmHelp,nil),
    NewStatusDef(hcHledejNahrad,hcHledejNahrad,
      NewStatusKey('~F1~ Npovda  Nahrazen zadanho etzce v editovanm souboru jinm',kbF1,cmHelp,nil),
    NewStatusDef(hcHledejZnovu,hcHledejZnovu,
      NewStatusKey('~F1~ Npovda  Optovn vykonn poslednho pkazu Hledej nebo Nahra',kbF1,cmHelp,nil),
    NewStatusDef(hcOkna,hcOkna,
      NewStatusKey('~F1~ Npovda  Pkazy pro prci s okny',kbF1,cmHelp,nil),
    NewStatusDef(hcOknaRozmer,hcOknaRozmer,
      NewStatusKey('~F1~ Npovda  Zmna rozmr a umstn okna',kbF1,cmHelp,nil),
    NewStatusDef(hcOknaZvetseni,hcOknaZvetseni,
      NewStatusKey('~F1~ Npovda  Zvten okna na maximln monou velikost',kbF1,cmHelp,nil),
    NewStatusDef(hcOknaMozaika,hcOknaMozaika,
      NewStatusKey('~F1~ Npovda  Uspodn oken do mozaiky',kbF1,cmHelp,nil),
    NewStatusDef(hcOknaStupne,hcOknaStupne,
      NewStatusKey('~F1~ Npovda  Uspodn oken do stup',kbF1,cmHelp,nil),
    NewStatusDef(hcOknaDalsi,hcOknaDalsi,
      NewStatusKey('~F1~ Npovda  Dal okno v poad se stane aktulnm',kbF1,cmHelp,nil),
    NewStatusDef(hcOknaPredchozi,hcOknaPredchozi,
      NewStatusKey('~F1~ Npovda  Pedchoz okno v poad se stane aktulnm',kbF1,cmHelp,nil),
    NewStatusDef(hcOknaUzavreni,hcOknaUzavreni,
      NewStatusKey('~F1~ Npovda  Uzaven aktulnho okna',kbF1,cmHelp,nil),
    NewStatusDef(hcDemo,hcDemo,
      NewStatusKey('~F1~ Npovda  Pkazy pro prci s demonstranmi okny',kbF1,cmHelp,nil),
    NewStatusDef(hcDJednoduche,hcDJednoduche,
      NewStatusKey('~F1~ Npovda  Demonstrace alg. pro gramatiku bez jednoduchch pravidel',kbF1,cmHelp,nil),
    NewStatusDef(hcDChomskeho,hcDChomskeho,
      NewStatusKey('~F1~ Npovda  Demonstrace alg. pro gramatiku v Chomskho normln form',kbF1,cmHelp,nil),
    NewStatusDef(hcDRekurze,hcDRekurze,
      NewStatusKey('~F1~ Npovda  Demonstrace alg. pro gramatiku bez lev rekurze',kbF1,cmHelp,nil),
    NewStatusDef(hcDGreibachova,hcDGreibachova,
      NewStatusKey('~F1~ Npovda  Demonstrace alg. pro gramatiku v Greibachov normln fomm',kbF1,cmHelp,nil),
    NewStatusDef(hcDVpred,hcDVpred,
      NewStatusKey('~F1~ Npovda  Krok vped v demonstrovanm algoritmu',kbF1,cmHelp,nil),
    NewStatusDef(hcDZpet,hcDZpet,
      NewStatusKey('~F1~ Npovda  Krok zpt v demonstrovanm algoritmu',kbF1,cmHelp,nil),
    NewStatusDef(hcDAuto,hcDAuto,
      NewStatusKey('~F1~ Npovda  Automatick krokovn a zastaven krokovn',kbF1,cmHelp,nil),
    NewStatusDef(hcDJdiNa,hcDJdiNa,
      NewStatusKey('~F1~ Npovda  Automatick krokovn a po zadan bod',kbF1,cmHelp,nil),
    NewStatusDef(hcDPreruseni,hcDPreruseni,
      NewStatusKey('~F1~ Npovda  Nastaven bod peruen',kbF1,cmHelp,nil),
    NewStatusDef(hcDNastaveni,hcDNastaveni,
      NewStatusKey('~F1~ Npovda  Nastaven maximlnho potu krok zpt',kbF1,cmHelp,nil),
    P))))))))))))))))))))))))))))))))))))))))))));
end;

procedure TEditorApp.OutOfMemory;
begin
  MessageBox('Nedostatek pamti pro vykonn tto operace.',
    nil, mfError + mfOkButton);
  Message(DeskTop,evBroadCast,cmSetFirst,nil);
  DrawView;
end;

procedure TEditorApp.Idle;
var P,Q : PView;

function IsTileable(P: PView): Boolean; far;
begin
  IsTileable := P^.Options and ofTileable <> 0;
end;

begin
  TApplication.Idle;
  Clock^.Update;
  if Desktop^.FirstThat(@IsTileable) <> nil then
    EnableCommands([cmTile, cmCascade])
  else
    DisableCommands([cmTile, cmCascade]);

  P:=PView(Message(DeskTop,evCommand,cmEditor,nil));
  if (P<>nil) and (P<>PView(ClipWindow)) then
    EnableCommands([cmTNadbytecne, cmTNedostupne, cmTRedukovana,
      cmTBezEpsilon, cmTCykly, cmTVlastni, cmTJednoduche,
      cmTBezRekurze, cmTChomskeho, cmTGreibachova, cmTZjednoduseni,
      cmReadGrammar, cmUkazStav, cmFind, cmReplace,
      cmSearchAgain, cmSave, cmSaveAs])
  else
    DisableCommands([cmTNadbytecne, cmTNedostupne, cmTRedukovana,
      cmTBezEpsilon, cmTCykly, cmTVlastni, cmTJednoduche,
      cmTBezRekurze, cmTChomskeho, cmTGreibachova, cmTZjednoduseni,
      cmReadGrammar, cmUkazStav, cmFind, cmReplace,
      cmSearchAgain, cmSave, cmSaveAs]);

  Q:=PView(Message(DeskTop,evCommand,cmDemoWindow,nil));
  if Q<>nil then
    EnableCommands([cmTraceAlg, cmGoTo, cmSetBreakPoint,
      cmDoStepF7, cmDoStepF8, cmUnDoStep])
  else
    DisableCommands([cmTraceAlg, cmGoTo, cmSetBreakPoint,
      cmDoStepF7, cmDoStepF8, cmUnDoStep]);

  if ((P<>nil) and (P<>PView(ClipWindow))) or (Q<>nil) then
    EnableCommands([cmDJednoduche, cmDChomskeho, cmDRekurze, cmDGreibachova,
    cmDoWrite])
  else
    DisableCommands([cmDJednoduche, cmDChomskeho, cmDRekurze, cmDGreibachova,
    cmDoWrite]);
end;

begin
  EditorApp.Init;
  EditorApp.Run;
  EditorApp.Done;
end.
