unit MyTracer;

{$X+}

interface

uses Objects, Views, Drivers, Tracer, ListSet, Rule, MyApp, Grammar;

const cmTraceAlg      = 142;
      cmGoToStep      = 143;
      cmGoTo          = 144;
      cmSetBreakPoint = 145;
      cmBreak         = 146;
      cmDemoWindow    = 147;
      cmGramatika     = 148;

const MaxNumOfBackSteps : word  = 10;

type
  PAlgWindow = ^TAlgWindow;
  TAlgWindow = object(TAlgWin)
      LastStep:word;
      Breakpoints:PCollection;
      Grammar:PGrammar;

      constructor Init(var Bounds:TRect;ATitle:TTitleStr;ANumber:integer;
                       HCC:word;st:word;DSize:word;g:PGrammar);
      destructor Done;virtual;
      procedure HandleEvent(var Event:TEvent);virtual;
      function  GetProperStep(st:word):integer;virtual;
      function  GetNumOfLines(st:word):byte;virtual;
      procedure DoStep(key:word);virtual;
      procedure UnDoStep;virtual;
      procedure ToStep(st,BreakKey:word);virtual;
      procedure SetLastStep(st:word);
      procedure GoToDialog;virtual;
      procedure BreakPointDialog;virtual;
  end;

  PAlgView = ^TAlgView;
  TAlgView = object(TAlgScroller)
    Color : byte;
    function  WriteSymbol(s:string):string;
    procedure WS(s:string;x,y,col:byte);
    procedure Ukaz(mnoz:PSet;uk:word;col:byte);
    procedure UkazSym(mnoz:PSet;n:word;col:byte);
    procedure Frame(title:string;fr:byte);
    procedure WP(pr:PRule;x,y,col:byte);
  end;

  PViewStartSym = ^TViewStartSym;
  TViewStartSym = object(TAlgView)
    procedure Draw;virtual;
  end;

procedure SetMaxNumOfBackSteps;

implementation

uses Dialogs, App, MsgBox, Common, DGHelp;

type

  PBreakPoint = ^TBreakPoint;
  TBreakPoint = record
      St,
      Off,
      Line,
      Pass,
      AktPass : word;
  end;

  PData = ^TData;
  TData = record
      Gramatika : PGrammar;
  end;

  PBreakCollection = ^TBreakCollection;
  TBreakCollection = object(TCollection)
      procedure FreeItem(Item: pointer); virtual;
  end;

  PBreakPointView = ^TBreakPointView;
  TBreakPointView = object(TListBox)
      constructor Init(var Bounds: TRect; ANumCols: word;
                       scBar: PScrollBar; col: PCollection);
      function GetText(Item:integer; MaxLen: integer):string;virtual;
  end;

  PBreakPointDialog = ^TBreakPointDialog;
  TBreakPointDialog = object(TDialog)
      Collection:PCollection;
      ScrollBar :PScrollBar;
      ListBox   :PListBox;
      AlgWindow :PAlgWindow;

      constructor Init(BreakPoints: PCollection; P:PAlgWindow);
      destructor Done;virtual;
      procedure HandleEvent(var Event:TEvent);virtual;
  end;

const
  cmOprav       = 2000;
  cmPridej      = 2001;
  cmZrus        = 2002;
  cmZrusVsechny = 2003;

constructor TAlgWindow.Init(var Bounds:TRect;ATitle:TTitleStr;ANumber:integer;
                 HCC:word;st:word;DSize:word;g:PGrammar);
begin
  TAlgWin.Init(Bounds,ATitle,ANumber,HCC,st,DSize);
  LastStep:=0;
  BreakPoints:=new(PBreakCollection,Init(0,1));
  Grammar:=g;
  if ANumber in [1..9] then WinNums:=WinNums+[ANumber];
  HelpCtx:=hcAlgWindow;
end;

destructor TAlgWindow.Done;
begin
  Message(Application,evBroadcast,cmBusy,nil);
  dispose(BreakPoints,Done);
  if Number in [1..9] then WinNums:=WinNums-[Number];
  TAlgWin.Done;
end;

procedure TAlgWindow.HandleEvent(var Event:TEvent);
begin
  TAlgWin.HandleEvent(Event);
  case Event.What of
    evCommand :
      case Event.Command of
        cmTraceAlg      : ToStep(LastStep,kbCtrlF9);
        cmGoToStep      : ToStep(Event.InfoWord,kbNoKey);
        cmGoTo          : GoToDialog;
        cmSetBreakPoint : BreakPointDialog;
        cmGramatika     : PGrammar(Event.InfoPtr^):=Grammar;
        cmDoWrite       : Message(Application,evCommand,cmWriteGrammar,Grammar);
        cmDemoWindow    : ;
        else exit;
      end;
    evBroadcast:
      case Event.Command of
        cmBreak         : MessageBox('Byl dosaen bod peruen',nil,
                                                  mfInformation+mfOkButton);
        else exit;
      end;
    else exit;
  end; 
  ClearEvent(Event);
end;

function  TAlgWindow.GetProperStep(st:word):integer;
begin
  if (st>=0) and (st<=LastStep) then GetProperStep:=st
  else GetProperStep:=-1;
end;

function  TAlgWindow.GetNumOfLines(st:word):byte;
begin
  if (st>=0) and (st<=LastStep) then GetNumOfLines:=1
  else GetNumOfLines:=0;
end;

procedure TAlgWindow.DoStep(key:word);
var i:integer;
    b:boolean;
    Event:TEvent;
begin
  Wait(500);
  i:=0;b:=false;
  with BreakPoints^ do
    while i<Count do begin
      with PBreakPoint(At(i))^ do 
        if Line=Step then begin
          inc(AktPass);
          if AktPass=Pass then b:=true;
        end;
      inc(i);
    end;
  if b then begin
    Message(@Self,evBroadCast,cmBreak,nil);
    with Event do begin
      What:=evCommand;
      Command:=cmRunStopAlg;
     end;
     PutEvent(Event);
   end;
end;

procedure TAlgWindow.UnDoStep;
var i:integer;
begin
  i:=0;
  with BreakPoints^ do
     while i<Count do begin
       with PBreakPoint(At(i))^ do
         if (Line=Step) and (AktPass>0) then dec(AktPass);
       inc(i);
     end;
  TAlgWin.UnDoStep;
  Wait(200);
end;

procedure TAlgWindow.ToStep(st,BreakKey:word);
var Event:TEvent;
begin
  Down:=true;
  CtrlBreakHit:=false;
  Message(@Self,evBroadcast,cmNewDown,Pointer(Down));
  repeat
    DoStep(kbF7);
    if EventAvail then GetEvent(Event);
  until (Step=st) or (Step=LastStep) or (Event.Command=cmRunStopAlg) or
        CtrlBreakHit or ((Event.KeyCode=BreakKey) and (BreakKey<>kbNoKey));
  CtrlBreakHit:=false;
end;

procedure TAlgWindow.SetLastStep(st:word);
begin
  LastStep:=st;
end;

procedure TAlgWindow.GoToDialog;
type TPom = record
       s1,s2:string[2];
     end;
const S:TPom=(s1:'1';s2:'1');
var c,c1,c2,n1,n2: integer;
    D: word;
    R: TRect;
    Dialog: PDialog;
    Control: PInputLine;
    Ok: boolean;
begin
  R.Assign(0, 0, 30, 9);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  Dialog:=new(PDialog,Init(R,'Clov krok'));
  with Dialog^ do begin
    HelpCtx:=hcCilovyKrok;
    R.Assign(16, 2, 20, 3);
    Control:=New(PInputLine, Init(R, 2));
    Insert(Control);
    R.Assign(2, 2, 15, 3);
    Insert(New(PLabel, Init(R, '~K~rok        :', Control)));

    R.Assign(16, 4, 20, 5);
    Control:=New(PInputLine, Init(R, 2));
    Insert(Control);
    R.Assign(2, 4, 15, 5);
    Insert(New(PLabel, Init(R, '~d~ek kroku :', Control)));

    R.Assign(2,6,10,8);
    Insert(New(PButton, Init(R, 'O~K~', cmYes, bfDefault)));
    R.Assign(11,6,21,8);
    Insert(New(PButton, Init(R, 'Zruit', cmCancel, bfNormal)));
    SelectNext(False);
  end;
  Dialog^.SetData(S);
  D := DeskTop^.ExecView(Dialog);
  Ok:=false;
  while (D<>cmCancel) and not Ok do begin
    Dialog^.GetData(S);
    with S do begin
      val(s1,c1,n1);
      val(s2,c2,n2);
      if (n1=0) and (n2=0) then begin
        c:=GetProperStep(c1);
        if (c>=0) and (c2>0) and (c2<=GetNumOfLines(c1))
        then Ok:=true;
      end;
    end;
    if not Ok then begin
      MessageBox('Chybn zadan hodnoty',nil,mfError+mfOkButton);
      D := DeskTop^.ExecView(Dialog);
    end;
  end;
  Dispose(Dialog, Done);
  if Ok then ToStep(c+c2-1,kbCtrlF9);
end;

procedure TAlgWindow.BreakPointDialog;
var Dialog:PDialog;
begin
  Dialog:=New(PBreakPointDialog, Init(BreakPoints,@Self));
  DeskTop^.ExecView(Dialog);
  Dispose(Dialog, Done);
end;

function  TAlgView.WriteSymbol(s:string):string;
begin
  with PData(DP)^.Gramatika^ do
    if s=epsilon then WriteSymbol:=EpsChar^.GetStr
    else if s=start then WriteSymbol:=StartSym^.GetStr
    else if (length(s)=1) and ((s=EpsChar^.GetStr) or
      (s[1] in EscChars))
    then WriteSymbol:=esc+s[1]
    else WriteSymbol:=s;
end;

procedure TAlgView.WS(s:string;x,y,col:byte);
var Buf:TDrawBuffer;
begin
  MoveChar(Buf,' ',col,Size.X);
  MoveStr(Buf,s,col);
  WriteLine(x,y,Size.X-x-1,1,Buf)
end;

procedure TAlgView.Ukaz(mnoz:PSet;uk:word;col:byte);
var l,m:word;
    od:integer;
begin
  if uk>0 then
    if (uk<=Delta.Y) or (uk>Delta.Y+Size.Y-2) then begin
      od:=uk - Size.Y div 2;
      if Size.Y>4 then
        if Down then inc(od)
        else dec(od);
      if od<0 then od:=0;
      Delta.Y:=od;
    end;

  for m:=1 to Size.Y-2 do
   with mnoz^ do begin
    l:=m+Delta.Y;
    if l<=Num then
      with GetItem(l)^ do
        if uk=l then WP(PRule(Pol),1,m,col)
        else WP(PRule(Pol),1,m,Color)
    else WS('',1,m,Color);
  end;
end;

procedure TAlgView.UkazSym(mnoz:PSet;n:word;col:byte);
var b:boolean;
    m,l,k:word;
    r,s:string;
    Buf:TDrawBuffer;
begin
  with mnoz^ do begin
    b:=true;
    MoveChar(Buf,' ',Color,Size.X);
    while b do begin
      m:=Delta.X+1;l:=1;k:=1;
      if (n=0) or (n>Num) then b:=false;
      while l<=Size.Y-2 do
        if m<=Num then begin
          r:='';
          s:=WriteSymbol(GetItem(m)^.GetStr);
          if m=n then begin
            MoveStr(Buf[k-1],s,col);
            inc(k,length(s)+1);
            MoveStr(Buf[k-2],',',Color);
            s:='';b:=false;
          end else s:=s+',';
          inc(m);
          while (length(r)+length(s)<Size.X-k-1) and (m<=Num) and (m<>n) do begin
            r:=WriteSymbol(GetItem(m)^.GetStr);
            if length(r)+length(s)<Size.X-k-1 then begin
              s:=s+r+',';
              inc(m);
            end;
          end;
          MoveStr(Buf[k-1],s,Color);
          inc(k,length(s));
          if (m<=Num) and (m=n) then begin
            if length(WriteSymbol(GetItem(m)^.GetStr))>=Size.X-k-1 then begin
              k:=1;
              WriteLine(1,l,Size.X-2,1,Buf);
              inc(l);
              MoveChar(Buf,' ',Color,Size.X);
            end
          end else begin
            k:=1;
            WriteLine(1,l,Size.X-2,1,Buf);
            MoveChar(Buf,' ',Color,Size.X);
            inc(l);
          end;
        end else begin
          WS('',1,l,Color);
          inc(l);
        end;
        Delta.Y:=m-Delta.X-1;
      if (n>0) and (n<=Num) and b then
        if Delta.Y=0 then b:=false
        else begin
          Delta.X:=n-1-Delta.Y div 2;
          if Delta.Y>3 then dec(Delta.X);
          if Delta.X<0 then Delta.X:=0;
        end;
    end;
  end;
end;

procedure TAlgView.Frame(title:string;fr:byte);
const f : array [0..1,1..6] of char =
          ('ͺɻȼ','ĳڿ');
var m:byte;
begin
  WriteChar(1,0,f[fr,1],1,Size.X-2);
  if Size.X>length(title)+1 then
    WriteStr((Size.X-length(title)-1)div 2+1,0,title,1);
  WriteChar(0,0,f[fr,3],1,1);
  WriteChar(Size.X-1,0,f[fr,4],1,1);

  for m:=1 to Size.Y-2 do begin
    WriteChar(0,m,f[fr,2],1,1);
    WriteChar(Size.X-1,m,f[fr,2],1,1);
  end;

  WriteChar(0,Size.Y-1,f[fr,5],1,1);
  WriteChar(1,Size.Y-1,f[fr,1],1,Size.X-2);
  WriteChar(Size.X-1,Size.Y-1,f[fr,6],1,1);
end;

procedure TAlgView.WP(pr:PRule;x,y,col:byte);
var i:word;
    s:string;
begin
  with pr^ do begin
    s:=WriteSymbol(Head^.GetStr)+' --> ';
    for i:=1 to Num do
      s:=s+WriteSymbol(GetItem(i)^.GetStr);
  end;
  WS(s,x,y,col);
end;

procedure TViewStartSym.Draw;
begin
  if First then begin
    Color:=GetColor(1);
    TAlgScroller.Draw;

    Frame('Start symbol',1);
    with PData(DP)^ do
      WS(Gramatika^.StartSym^.GetStr,1,1,GetColor(3));
  end;
end;

procedure TBreakCollection.FreeItem(Item: pointer);
begin
  if Item<>nil then dispose(PBreakPoint(Item));
end;

constructor TBreakPointDialog.Init(BreakPoints: PCollection; P: PAlgWindow);
var R: TRect;
begin
  R.Assign(0,0,61,16);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  TDialog.Init(R, 'BreakPointy');
  Collection:=BreakPoints;
  AlgWindow:=P;
  HelpCtx:=hcBodPreruseni;
  R.Assign(58,4,59,12);
  ScrollBar:=new(PScrollBar,Init(R));
  Insert(ScrollBar);
  R.Assign(3,4,58,12);
  ListBox:=new(PBreakPointView,Init(R,1,ScrollBar,Collection));
  Insert(ListBox);
  R.Assign(3,2,58,3);
  Insert(New(PLabel,Init(R,
         '    Krok         dek         Prchod      Vykonno   ',
         ListBox)));
  R.Assign(2, 13, 11, 15);
  Insert(New(PButton, Init(R, '~O~prav', cmOprav, bfNormal)));
  R.Assign(13, 13, 23, 15);
  Insert(New(PButton, Init(R, '~P~idej', cmPridej, bfNormal)));
  R.Assign(25, 13, 33, 15);
  Insert(New(PButton, Init(R, '~Z~ru', cmZrus, bfNormal)));
  R.Assign(34, 13, 50, 15);
  Insert(New(PButton, Init(R, 'Zru ~v~echny', cmZrusVsechny, bfNormal)));
  R.Assign(51, 13, 59, 15);
  Insert(New(PButton, Init(R, 'O~K~', cmOk, bfDefault)));
  SelectNext(false);
end;

destructor TBreakPointDialog.Done;
begin
  dispose(ScrollBar,Done);
  TDialog.Done;
end;

procedure TBreakPointDialog.HandleEvent(var Event:TEvent);
var br:PBreakPoint;

 function Edituj(b:PBreakPoint):boolean;
 type TPom=record
        s1,s2:string[2];
        s3:string[3];
      end;

 var R:TRect;
     Dialog:PDialog;
     Control:PView;
     S:TPom;
     ok:boolean;
     c,c1,c2,c3,n1,n2,n3:integer;
 begin
   R.Assign(0,0,23,11);
   R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
   Dialog:=new(PDialog,Init(R,'Zadn bodu peruen'));
   with Dialog^ do begin
     HelpCtx:=hcJedenKrok;
     R.Assign(16, 2, 20, 3);
     Control:=New(PInputLine, Init(R, 2));
     Insert(Control);
     R.Assign(2, 2, 15, 3);
     Insert(New(PLabel, Init(R, '~K~rok        :', Control)));

     R.Assign(16, 4, 20, 5);
     Control:=New(PInputLine, Init(R, 2));
     Insert(Control);
     R.Assign(2, 4, 15, 5);
     Insert(New(PLabel, Init(R, '~d~ek kroku :', Control)));

     R.Assign(16, 6, 21, 7);
     Control:=New(PInputLine, Init(R, 3));
     Insert(Control);
     R.Assign(2, 6, 15, 7);
     Insert(New(PLabel, Init(R, '~P~rchody    :', Control)));

     R.Assign(2,8,10,10);
     Insert(New(PButton, Init(R, 'O~K~', cmYes, bfDefault)));
     R.Assign(11,8,21,10);
     Insert(New(PButton, Init(R, 'Zruit', cmCancel, bfNormal)));
     SelectNext(False);
   end;
   with S do
     with b^ do begin
       str(St:2,s1);
       str(Off:2,s2);
       str(Pass:3,s3);
     end;
   Dialog^.SetData(S);
   C := DeskTop^.ExecView(Dialog);
   Ok:=false;
   while (C<>cmCancel) and not Ok do begin
     Dialog^.GetData(S);
     with S do
       with b^ do begin
         val(s1,c1,n1);
         val(s2,c2,n2);
         val(s3,c3,n3);
         if (n1=0) and (n2=0) and (n3=0) then begin
           c:=AlgWindow^.GetProperStep(c1);
           if (c>=0) and (c2>0) and
              (c2<=AlgWindow^.GetNumOfLines(c1)) and (c3>0)
           then begin
             St:=c1;
             Off:=c2;
             Line:=c+c2-1;
             Pass:=c3;
             AktPass:=0;
             Ok:=true;
           end;
         end;
       end;
     if not Ok then begin
       MessageBox('Chybn zadan hodnoty',nil,mfError+mfOkButton);
       C := DeskTop^.ExecView(Dialog);
     end;
   end;
   Edituj:=Ok;
   Dispose(Dialog, Done);
 end;

begin
  TDialog.HandleEvent(Event);
  if Event.What=evCommand then begin
    case Event.Command of
      cmOprav : with Collection^ do
                  if ListBox^.Focused<Count then
                    Edituj(PBreakPoint(At(ListBox^.Focused)));
      cmPridej : with Collection^ do begin
                   new(br);
                   with br^ do begin
                     St:=1;Off:=1;Line:=1;Pass:=1;AktPass:=0;
                   end;
                   if Edituj(br) then 
                     if Count>0 then AtInsert(ListBox^.Focused+1,br)
                     else Insert(br)
                   else dispose(br);
                 end;
      cmZrus : with ListBox^ do begin
                 if Focused<Collection^.Count
                 then with Collection^ do Free(At(Focused));
               end;
      cmZrusVsechny : Collection^.FreeAll;
      else exit;
    end;
    ListBox^.SetRange(Collection^.Count);
    ListBox^.DrawView;
    ClearEvent(Event);
  end;
end;

constructor TBreakPointView.Init(var Bounds: TRect; ANumCols: word;
                                 scBar: PScrollBar; col: PCollection);
var R:TRect;
begin
  TListBox.Init(Bounds,ANumCols,scBar);
  Options:=Options or ofPostProcess;
  NewList(col);
end;

function TBreakPointView.GetText(Item:integer; MaxLen: integer):string;
var r,s:string;
    i:integer;
begin
  GetText:='                                                  ';
  if (Item>=0) and (Item<List^.Count) then
    with PBreakPoint(List^.At(Item))^ do begin
      str(St:2,s);
      r:='     '+s;
      str(Off:2,s);
      r:=r+'         '+s;
       str(Pass:3,s);
      r:=r+'              '+s;
      str(AktPass:3,s);
      r:=r+'           '+s;
    end;
  GetText:=r+'     ';
end;

procedure SetMaxNumOfBackSteps;
const S:string[3]='10';
var c,n: integer;
    D: word;
    R: TRect;
    Dialog: PDialog;
    Control: PInputLine;
    Ok: boolean;
begin
  R.Assign(0, 0, 42, 7);
  R.Move((Desktop^.Size.X - R.B.X) div 2, (Desktop^.Size.Y - R.B.Y) div 2);
  Dialog:=new(PDialog,Init(R,'Nastaven'));
  with Dialog^ do begin
    HelpCtx:=hcNastaveni;
    R.Assign(35, 2, 40, 3);
    Control:=New(PInputLine, Init(R, 3));
    Insert(Control);
    R.Assign(2, 2, 34, 3);
    Insert(New(PLabel, Init(R, '~M~aximln poet zptnch krok :', Control)));

    R.Assign(18,4,28,6);
    Insert(New(PButton, Init(R, 'O~K~', cmYes, bfDefault)));
    R.Assign(30,4,40,6);
    Insert(New(PButton, Init(R, 'Zruit', cmCancel, bfNormal)));
    SelectNext(False);
  end;
  Dialog^.SetData(S);
  D := DeskTop^.ExecView(Dialog);
  Ok:=false;
  while (D<>cmCancel) and not Ok do begin
    Dialog^.GetData(S);
    val(S,c,n);
    if (n=0) and (c>=0) and (c<=200) then Ok:=true;
    if not Ok then begin
      MessageBox('Povolen rozmez je 0 - 200',nil,mfError+mfOkButton);
      D := DeskTop^.ExecView(Dialog);
    end;
  end;
  Dispose(Dialog, Done);
  if Ok then MaxNumOfBackSteps:=c;
end;

end.